/* AritStrs.c
 * Mdulo de aritmtica de enteros con cadenas.
 */
#include <string.h>
#include "AritStrs.h"

/* Funciones locales.
 * Como no queremos exportarlas anteponemos la palabra static a su
 * definicin. As slo son visibles dentro de este programa y se pueden
 * tener funciones con el mismo nombre en otros mdulos */
static int  Digito (const char *num, int pos);
static void DesplazaDcha (char *num);

/* Sumador de nmeros de longitud arbitraria expresados como cadena de
 * caracteres.
 *
 * Se supone que ambos operandos contienen una cadena positiva, es decir,
 * que no tienen signo. */
int SumaStrs (char res[], const char nu1[], const char nu2[])
{
  int l1, l2, l3, l4; /* Longitudes de las cadenas */
  int llevo;          /* Llevada de la suma */
  int s;              /* Suma parcial de dos dgitos ms llevada */

  l1 = strlen (nu1);        /* l1: longitud de nu1 */
  l2 = strlen (nu2);        /* l2: longitud de nu2 */
  l4 = l3 = l1 > l2? l1:l2; /* l3 y l4: el mayor de entre l1 y l2 */
  res [l3] = '\0';          /* Ponemos el terminador a la cadena resultante */
  llevo = 0;                /* Al principio no hay llevada en la suma */

  while (l3 > 0) { /* Vamos de derecha a izquierda */
    s = Digito (nu1, --l1) + Digito (nu2, --l2) + llevo; /* Suma parcial */
    res [--l3] = '0' + s % 10; /* Unidades de s convertidas a letra */
    llevo = s / 10;            /* La llevada son las decenas de s */
  }

  if (llevo) {
    if (l4 == TOPE) /* l4 es la longitud de la cadena resultante */
      return 1; /* Sobrepasamiento: no me cabe la llevada en res */
    DesplazaDcha (res); /* Si no hay sobrepasamiento desplazar */
    res [0] = '1';      /*   e insertar la llevada */
  }
  return 0; /* Si llego aqu quiere decir que no hay sobrepasamiento */
}

/* Devuelve el dgito (int) correspondiente al carcter en la posicin pos,
 * considerando un cero a la izquierda cuando se hace referencia a una
 * posicin negativa.
 *
 * Ejemplos:
 * - Digito ("46",  0) -> 4
 * - Digito ("46",  1) -> 6
 * - Digito ("46", -2) -> 0 (dos ceros a la izquierda 0046) */
static int Digito (const char *num, int pos)
{
  if (pos < 0)
    return 0;
  return num [pos] - '0'; /* Ej. '1' - '0' -> 1 ya que 31 - 30 -> 1 */
}

/* Desplaza una cadena a la derecha, manteniendo el primer carcter.
 * Si antes contena "Hola" luego contendr "HHola" */
static void DesplazaDcha (char *num)
{
  int lon;

  lon = strlen (num);
  while (lon >= 0) {
    num [lon + 1] = num [lon];
    lon--;
  }
}

/* Prototipo de otra funcin local */
static void EliminaCerosIzda (char *num);

/* Restador de nmeros de longitud arbitraria expresados como cadena de
 * caracteres.
 *
 * Se supone que ambos operandos contienen una cadena positiva, es decir,
 * que no tienen signo.  */
int SubsStrs (char *res, const char *nu1, const char *nu2)
{
  int l1, l2, tmp;
 const char *n1, *n2;
  int r;
  int signo;
  int debo;

  l1 = strlen (nu1);
  l2 = strlen (nu2);
  if      (l1 > l2)               signo = 0;
  else if (l2 > l1)               signo = 1;
  else if (strcmp (nu1, nu2) < 0) signo = 1;
  else                            signo = 0;
  if (signo) {
    n1 = nu2;
    n2 = nu1;
      /* Intercambiar longitudes */
    tmp = l1;
    l1 = l2;
    l2 = tmp;
  }
  else {
    n1 = nu1;
    n2 = nu2;
  }
  res [l1] = '\0'; /* Ponemos el terminador a la cadena resultante */
  debo = 0;        /* Al principio no debo nada en la resta */

  while (l1 > 0) {
    r = Digito (n1, --l1) - Digito (n2, --l2) - debo;
    if (r < 0) {
      r += 10;
      debo = 1;
    }
    else
      debo = 0;
    res [l1] = '0' + r % 10; /* Unidades de r convertidas a letra */
  }

  EliminaCerosIzda (res);

  return signo;
}

  /* Eliminar todos los ceros a la izquierda (menos el ltimo si es 0) */
static void EliminaCerosIzda (char *num)
{
  int n0; /* Nmero de ceros a la izquierda */
  int nd; /* Nmero de dgitos significativos */

  if (num [0] == '0') { /* Si el primer carcter es '0' */
      /* Contar ceros */
    n0 = 0;
    while (num [n0] == '0' && num [n0] != '\0')
      n0++;
    if (num [n0] == '\0') /* Si todos son ceros */
      num [1] = '\0';
    else { /* copiar los digitos significativos sobreescribiendo los ceros */
      nd = 0;
      while (num [n0+nd] != '\0')
        num [nd++] = num [n0+nd];
      num [nd] = '\0';
    }
  }
}

int SumaConSigno (char *res, const char *nu1, const char *nu2)
{
  int signo;

  if (*nu1 != '-' && *nu2 != '-')    /* Caso 1 */
    return SumaStrs (res, nu1, nu2); /* La funcin termina */

  if (*nu1 == '-' && *nu2 == '-') {              /* Caso 2 */
    *res = '-';
    return SumaStrs (&res[1], &nu1[1], &nu2[1]); /* La funcin termina */
  }

  if (*nu1 == '-') signo = SubsStrs (res, nu2, &nu1[1]); /* Caso 3 */
  else             signo = SubsStrs (res, nu1, &nu2[1]); /* Caso 4 */
  if (signo) {
    DesplazaDcha (res);
    *res = '-';
  }

  return 0;
}

int SubsConSigno (char res[], const char nu1[], const char nu2[])
{
  int signo;

  if (*nu1 != '-' && *nu2 == '-')            /* Caso 1 */
    return SumaStrs (res, nu1, &nu2[1]);     /* La funcin termina */

  if (*nu1 == '-' && *nu2 != '-') {          /* Caso 2 */
    *res = '-';
    return SumaStrs (&res[1], &nu1[1], nu2); /* La funcin termina */
  }

  if (*nu1 != '-') signo = SubsStrs (res, nu1, nu2);         /* Caso 3 */
  else             signo = SubsStrs (res, &nu1[1], &nu2[1]); /* Caso 4 */
  if (signo) {
    DesplazaDcha (res);
    *res = '-';
  }

  return 0;
}
